@@ -15,6 +15,14 @@ typedef struct conf_data {
 int get_config _PROTO(( char * conf_file, conf_data * cd,
 		        char * section_key, char * key_value ));
 
+/* some versions of BSD have their own variant of fgetline that
+ * behaves differently. Just change the name for now...
+ * FIXME.
+ */
+#ifdef BSD
+# define fgetline mgetty_fgetline
+#endif
+
 char * fgetline _PROTO(( FILE * fp ));
 void   norm_line _PROTO(( char ** line, char ** key ));
 
