@@ -4,6 +4,10 @@
 #include <unistd.h>
 #include <sys/times.h>
 
+#ifdef ISC
+#include <sys/bsdtypes.h>
+#endif
+
 #include "mgetty.h"
 #include "policy.h"
 
@@ -22,12 +26,13 @@ char * CallName = "unknown";	/* ditto */
 int main _P2( (argc, argv), int argc, char ** argv )
 {
 char * fax_spool_in = FAX_SPOOL_IN;
-int c;
+int  c;
+char log_path[MAXPATH];
 
     while ((c = getopt(argc, argv, "x:d:")) != EOF) {
 	switch (c) {
 	case 'x':
-		log_level = atoi(optarg);
+		log_set_llevel( atoi(optarg) );
 		break;
 	case 'd':
 		fax_spool_in = optarg;
@@ -52,6 +57,9 @@ int c;
     /* construct the log path string */
     sprintf( log_path, LOG_PATH, Device );
 
+    /* initialize logging subsystem */
+    log_init_paths( argv[0], log_path, &Device[strlen(Device)-2] );
+
 
     /* receive the fax */
     faxrec( fax_spool_in );
