@@ -1,4 +1,4 @@
-#ident "@(#)mgetty.c	1.88 94/01/23 Copyright (c) Gert Doering"
+#ident "@(#)mgetty.c	1.102 94/04/21 Copyright (c) Gert Doering"
 ;
 /* mgetty.c
  *
@@ -34,6 +34,8 @@
 #include "voclib.h"
 #endif
 
+#include "mg_utmp.h"
+
 unsigned short portspeed = B0;	/* indicates has not yet been set */
 
 /* warning: some modems (for example my old DISCOVERY 2400P) need
@@ -62,6 +64,9 @@ char *	init_chat_seq[] = { "", "\\d\\d\\d+++\\d\\d\\d\r\\dATQ0V1H0", "OK",
 			    /*"AT+FLID=\""FAX_STATION_ID"\"", "OK",*/
 			    "AT+FDCC=1,5,0,2,0,0,0", "OK",
 #endif
+#ifdef DIST_RING
+			    DIST_RING_INIT, "OK",
+#endif
 #ifdef VOICE
 			    "AT+FCLASS=8", "OK",
 #endif
@@ -75,7 +80,13 @@ chat_action_t	init_chat_actions[] = { { "ERROR", A_FAIL },
 					{ NULL,	A_FAIL } };
 
 int	rings_wanted = 1;		/* default: one "RING" */
+
+#ifdef DIST_RING
+char *	ring_chat_seq[] = { "RING\r", NULL };
+#else
 char *	ring_chat_seq[] = { "RING", NULL };
+#endif
+
 chat_action_t	ring_chat_actions[] = { { "CONNECT",	A_CONN },
 					{ "NO CARRIER", A_FAIL },
 					{ "BUSY",	A_FAIL },
@@ -86,13 +97,24 @@ chat_action_t	ring_chat_actions[] = { { "CONNECT",	A_CONN },
 #endif
 #ifdef VOICE
 					{ "VCON",       A_VCON },
+#endif
+#ifdef DIST_RING
+					{ "RING 1",	A_RING1 },
+					{ "RING 2",	A_RING2 },
+					{ "RING 3",	A_RING3 },
+					{ "RING 4",	A_RING4 },
+					{ "RING 5",	A_RING5 },
 #endif
 					{ NULL,		A_FAIL } };
 
 #ifdef VOICE
 char *	answer_chat_seq[] = { "", VOICE_ATA, "VCON", NULL };
 #else
+# ifdef ELINK
+char *	answer_chat_seq[] = { "", "AT\\\\OA", "CONNECT", "\\c", "\n", NULL };
+# else
 char *	answer_chat_seq[] = { "", "ATA", "CONNECT", "\\c", "\n", NULL };
+# endif /* !ELINK */
 #endif
 
 int	answer_chat_timeout = 80;
@@ -146,7 +168,8 @@ static RETSIGTYPE sig_pick_phone()		/* "simulated RING" handler */
 }
 static RETSIGTYPE sig_goodbye _P1 ( (signo), int signo )
 {
-    lprintf( L_AUDIT, "failed, pid=%d, got signal %d, exiting", getpid(), signo );
+    lprintf( L_AUDIT, "failed dev=%s, pid=%d, got signal %d, exiting",
+	              Device, getpid(), signo );
     rmlocks();
     exit(10);
 }
@@ -195,6 +218,16 @@ int main _P2((argc, argv), int argc, char ** argv)
 	(void) signal(SIGQUIT, SIG_DFL);
 	(void) signal(SIGTERM, SIG_DFL);
 
+	/* some systems, notable BSD 4.3, have to be told that system
+	 * calls are to be interrupted by signals.
+	 */
+#ifdef HAVE_SIGINTERRUPT
+	siginterrupt( SIGINT,  TRUE );
+	siginterrupt( SIGALRM, TRUE );
+	siginterrupt( SIGHUP,  TRUE );
+	siginterrupt( SIGUSR1, TRUE );
+#endif
+
 	Device = "unknown";
 
 	/* process the command line
@@ -215,7 +248,7 @@ int main _P2((argc, argv), int argc, char ** argv)
 			minfreespace = atol(optarg);
 			break;
 		case 'x':			/* log level */
-			log_level = atoi(optarg);
+			log_set_llevel( atoi(optarg) );
 			break;
 		case 's':			/* port speed */
 			cspeed = atoi(optarg);
@@ -272,7 +305,8 @@ int main _P2((argc, argv), int argc, char ** argv)
 	sprintf(devname, "/dev/%s", Device);
 
 	/* name of the logfile is device-dependant */
-	sprintf( log_path, LOG_PATH, Device );
+	sprintf( buf, LOG_PATH, strrchr( devname, '/' ) + 1 );
+	log_init_paths( argv[0], buf, &devname[strlen(devname)-2] );
 
 #ifdef USE_GETTYDEFS
 	if (optind < argc)
@@ -329,6 +363,11 @@ int main _P2((argc, argv), int argc, char ** argv)
 		exit(FAIL);
 	}
 
+	/* unset O_NDELAY (otherwise waiting for characters */
+	/* would be "busy waiting", eating up all cpu) */
+
+	fcntl( fd, F_SETFL, O_RDWR);
+
 	/* make new fd == stdin if it isn't already */
 
 	if (fd > 0) {
@@ -361,11 +400,6 @@ int main _P2((argc, argv), int argc, char ** argv)
 	setbuf(stdout, (char *) NULL);
 	setbuf(stderr, (char *) NULL);
 
-	/* unset O_NDELAY (otherwise waiting for characters */
-	/* would be "busy waiting", eating up all cpu) */
-
-	fcntl(STDIN, F_SETFL, O_RDWR);
-
 	/* setup terminal */
 
 	/* Currently, the tio returned here is ignored.
@@ -391,6 +425,7 @@ int main _P2((argc, argv), int argc, char ** argv)
 	}
 	tio_mode_sane( &tio, TRUE );
 	tio_set_speed( &tio, portspeed );
+	tio_default_cc( &tio );
 	tio_mode_raw( &tio );
 #ifdef sun
 	/* sunos does not rx with RTSCTS unless carrier present */
@@ -446,11 +481,11 @@ int main _P2((argc, argv), int argc, char ** argv)
 
 	signal( SIGUSR1, sig_pick_phone );
 
-#if defined(linux) && !defined(NO_SYSVINIT)
+#if defined(linux) && defined(NO_SYSVINIT)
 	/* on linux, "simple init" does not make a wtmp entry when you
 	 * log so we have to do it here (otherwise, "who" won't work)
 	 */
-	make_utmp_wtmp( Device, FALSE );
+	make_utmp_wtmp( Device, UT_INIT, "uugetty" );
 #endif
 
 #ifdef VOICE
@@ -482,6 +517,10 @@ waiting:
 	    close(1);
 	    close(2);
 
+	    /* write a note to utmp/wtmp about dialout
+	     */
+	    make_utmp_wtmp( Device, UT_USER, "dialout" );
+
 	    /* this is kind of tricky: sometimes uucico dial-outs do still
 	       collide with mgetty. So, when my uucico times out, I do
 	       *immediately* restart it. The double check makes sure that
@@ -495,7 +534,7 @@ waiting:
 		    /* wait a moment, then check for reappearing locks */
 		    sleep(5);
 	    }
-	    while ( checklock(Device) == TRUE );	
+ 	    while ( checklock(Device) == TRUE );	
 
 	    /* OK, leave & get restarted by init */
 	    exit(0);
@@ -511,13 +550,26 @@ waiting:
 #ifdef NOLOGIN_FILE
 	/* check for a "nologin" file (/etc/nologin.<device>) */
 	
-	sprintf( buf, NOLOGIN_FILE, Device );
+	sprintf( buf, NOLOGIN_FILE, strrchr( devname, '/' )+1 );
 
 	if ( access( buf, F_OK ) == 0 )
 	{
 	    lprintf( L_MESG, "%s exists - do not accept call!", buf );
 	    clean_line( STDIN, 80 );		/* wait for ringing to stop */
-	    exit(1);
+
+	    /* and return to state waiting. If it was a data or fax
+	     * call, you can now press DATA/VOICE and have the modem
+	     * manually pickup the phone
+	     *
+	     * WARNING: if you press the button too soon, or if the
+	     * modem auto-answers, this will fail. FIXME. Use do_chat()
+	     * here (and count the RINGs for logging...)
+	     */
+	    lprintf( L_AUDIT, "rejected" );
+	    rmlocks();
+	    goto waiting;
+	       
+	    /* exit(1); */
 	}
 #endif
     
@@ -536,15 +588,24 @@ waiting:
 	    {
 		if ( do_chat( STDIN, ring_chat_seq, ring_chat_actions,
 			      &what_action, ring_chat_timeout,
-			      TRUE ) == FAIL ) break;
+			      TRUE ) == FAIL 
+#ifdef DIST_RING
+		    && (what_action != DIST_RING_VOICE)
+#endif
+		   ) break;
 		rings++;
 	    }
 
 	    /* timeout - the phone stopped ringing? (human picked up) */
 	    if ( rings < rings_wanted && what_action == A_TIMOUT )
 	    {
-		lprintf( L_MESG, "phone stopped ringing" );
 		rmlocks();		/* free line again */
+		if ( rings == 0 )	/* no ring *at all* */
+		{
+		    lprintf( L_WARN, "huh? Junk on the line?" );
+		    exit(0);		/* let init restart mgetty */
+		}
+		lprintf( L_MESG, "phone stopped ringing" );
 		goto waiting;
 	    }
 
@@ -571,6 +632,9 @@ waiting:
 
 	    if ( what_action != A_CONN &&
 		 what_action != A_VCON &&	/* vgetty extensions */
+#ifdef DIST_RING
+		 (what_action < A_RING1 || what_action > A_RING5) &&
+#endif
 		 ( rings < rings_wanted ||
 	           do_chat( STDIN, answer_chat_seq, answer_chat_actions,
 			    &what_action, answer_chat_timeout, TRUE) == FAIL))
@@ -611,21 +675,25 @@ waiting:
 	    voice_answer(rings, rings_wanted,
 			 answer_chat_actions,
 			 answer_chat_timeout,
-			 answer_mode );
+			 answer_mode,
+			 what_action );
 	}
 #endif /* VOICE */
 
-	/* make utmp and wtmp entry (otherwise login won't work)
-	 */
-	make_utmp_wtmp( Device, TRUE );
-
-        delay( prompt_waittime );
-
 	/* honor carrier now: terminate if modem hangs up prematurely
 	 */
 	tio_carrier( &tio, TRUE );
 	tio_set( STDIN, &tio );
-	
+
+	/* make utmp and wtmp entry (otherwise login won't work)
+	 */
+	make_utmp_wtmp( Device, UT_LOGIN, "LOGIN" );
+
+	/* wait a little bit befor printing login: prompt (to give
+	 * the other side time to get ready)
+	 */
+        delay( prompt_waittime );
+
 	/* loop until a successful login is made
 	 */
 	for (;;)
@@ -642,16 +710,19 @@ waiting:
 
 		/* display ISSUE, if present
 		 */
-		if (*issue != '/') {
-			fputs(issue, stdout);
-			fputs("\r\n", stdout);
-		} else if ((fp = fopen(issue, "r")) != (FILE *) NULL) {
-			while (fgets(buf, sizeof(buf), fp) != (char *) NULL)
-			{
-				fputs(buf, stdout);
-				fputc('\r', stdout );
-			}
-			fclose(fp);
+		if (*issue != '/')
+		{
+		    printf( "%s\r\n", ln_escape_prompt( issue ) );
+		}
+		else if ((fp = fopen(issue, "r")) != (FILE *) NULL)
+		{
+		    while (fgets(buf, sizeof(buf), fp) != (char *) NULL)
+		    {
+			char * p = ln_escape_prompt( buf );
+			if ( p != NULL ) fputs( p, stdout );
+			fputc('\r', stdout );
+		    }
+		    fclose(fp);
 		}
 
 		/* set permissions to "rw-------" */
