@@ -1,4 +1,4 @@
-#ident "@(#)mksed.c	1.3 94/01/30 Copyright (c) Gert Doering"
+#ident "@(#)mksed.c	1.6 94/04/25 Copyright (c) Gert Doering"
 ;
 #include <stdio.h>
 
@@ -16,7 +16,10 @@ int main _P0( void )
     printf( "      -e 's;@FAX_LOG;%s;g'\\\n", FAX_LOG );
     printf( "      -e \"s;@MAILER@;%s;g\"\\\n", MAILER );
     printf( "      -e 's;@FAX_ADMIN@;%s;g'\\\n", MAIL_TO );
+    printf( "      -e 's;@AWK@;%s;g'\\\n", AWK );
     printf( "      -e 's;@BINDIR@;%s;g'\\\n", BINDIR );
-    printf( "      -e 's;@LIBDIR@;%s;g'\n", LIBDIR );
+    printf( "      -e 's;@SBINDIR@;%s;g'\\\n", SBINDIR );
+    printf( "      -e 's;@LIBDIR@;%s;g'\\\n", LIBDIR );
+    printf( "      -e 's;@VOICE_DIR@;%s;g'\n", VOICE_DIR );
     return 0;
 }
