@@ -1,4 +1,4 @@
-/* $Header: /home/klaus/mgetty/voice/RCS/pvfmain.c,v 1.6 1994/01/31 17:21:49 klaus Exp $ */
+/* $Header: /home/klaus/mgetty/voice/RCS/pvfmain.c,v 1.7 1994/04/22 19:58:04 klaus Exp $ */
 /* The dispatcher for the pvf utilities. It looks up the name in
  * argv[0] and calls the appropiate subroutine.
  */
@@ -13,7 +13,7 @@
 
 typedef int pvffunc _PROTO((int, char**));
 
-extern pvffunc pvfamp, pvfcut, pvfecho,
+extern pvffunc pvfamp, pvfcut, pvfecho, pvfsine,
                pvfreverse, pvfspeed, pvfmix, pvfadpcm,
                pvftoau, autopvf, pvftobasic, basictopvf,
                pvftolin, lintopvf, pvftovoc, voctopvf;
@@ -30,7 +30,8 @@ static tab_t tab[]={
 	{"pvfreverse", pvfreverse},
 	{"pvfspeed", pvfspeed},
 	{"pvfmix", pvfmix},
-
+	{"pvfsine", pvfsine},
+	    
 	{"pvftoadpcm2", pvfadpcm},
 	{"pvftoadpcm3", pvfadpcm},
 	{"adpcmtopvf",  pvfadpcm},
